// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: catmap_util.cc 741 2007-08-14 02:39:19Z ctsa $

/// \file

#include "cat_info.h"
#include "catmap_util.h"
#include "util/general/log.h"

#include <cstdlib>

#include <algorithm>
#include <ostream>


void
catmap_to_catlup(const site_data& sd,
                 const unsigned n_assigned_cats,
                 const data_catmap_type& catmap,
                 std::vector<unsigned>& catlup){

  catlup.resize(sd.max_cat()+1);

  if(catmap.empty()){
    std::fill(catlup.begin(),catlup.end(),UNASSIGNED_CAT_ID);
  } else {
    std::fill(catlup.begin(),catlup.end(),n_assigned_cats+1);

    data_catmap_type::const_iterator i=catmap.begin(),i_end=catmap.end();
    for(;i!=i_end;++i){
      const std::string& data_cat_label(i->first);
      const unsigned model_cat_num(i->second);
      if( sd.catlabel_2_id.find(data_cat_label) != sd.catlabel_2_id.end() ){
        const unsigned data_cat_num(sd.catlabel_2_id.find(data_cat_label)->second);
        catlup[data_cat_num]=model_cat_num;
      } else {
        log_os << "ERROR:: can't find mapped data category: " << data_cat_label << "\n";
        exit(EXIT_FAILURE);
      }
    }
  }
}


unsigned
assigned_cat_size(const data_catmap_type& catmap) {
  unsigned acs(0);
  data_catmap_type::const_iterator i(catmap.begin());
  const data_catmap_type::const_iterator i_end(catmap.end());
  for(;i!=i_end;++i){
    const unsigned assigned_cat_num(i->second);
    acs=std::max(assigned_cat_num,acs);
  }
  return acs;
}


bool
is_unassigned_cat(const data_catmap_type& catmap){
  data_catmap_type::const_iterator i(catmap.begin());
  const data_catmap_type::const_iterator i_end(catmap.end());
  if(i==i_end) return true;
  for(;i!=i_end;++i){
    if(i->second==UNASSIGNED_CAT_ID) return true;
  }
  return false;
}

