// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: lhood_model.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __LHOOD_MODEL_H
#define __LHOOD_MODEL_H

#include "simple_util.h"
#include "subs_ml_types.h"

#include <memory>
#include <vector>


struct condition_func;
struct site_data_fastlup;
struct submodel_manager;
struct subs_ml_model;
struct tree_probs;
struct workspace;


smlfloat
get_lnp_norm(const subs_ml_model& mdl,
             const site_data_fastlup& sdf);

// all data that can be initialized and/or malloced independent of
// mdl's parameter state, such that these initializations can be
// called once before many lnprob calls where only the model
// parameters are allowed to change, (as in during minimization)
//
struct nonparam_lnprob_data {

  nonparam_lnprob_data(const subs_ml_model& mdl,
                       const site_data_fastlup& sdf);

  ~nonparam_lnprob_data();

private:
  nonparam_lnprob_data(const nonparam_lnprob_data&);
  nonparam_lnprob_data& operator=(const nonparam_lnprob_data&);

public:
  condition_func& cf() { return *_cf; }
  submodel_manager& sm() { return *_sm; }
  tree_probs& tprob() { return *_tprob; }

private:
  std::auto_ptr<condition_func> _cf;
  std::auto_ptr<submodel_manager> _sm;
  std::auto_ptr<tree_probs> _tprob;

public:
  prob_t* cat_site_prob;
  prob_t* unassigned_cat_site_prob;

  std::vector<prob_t> site_cat_prior_pdistro;
  std::vector<prob_t> group_cat_prior_pdistro;

  // ppit mode extra's
  simple_init_matrix<prob_t> up_branch_prob;
  simple_init_matrix<prob_t> down_branch_prob;
  simple_init_array<prob_t> tmp_state_array;

  workspace* tprob_ws;
};


/// get prob of data given model
///
/// requires a handle to parameter independent data (nlp)
///
void
get_lnprob_from_param_prep(const subs_ml_model& mdl,
                           const site_data_fastlup& full_sdf,
                           nonparam_lnprob_data& nlp,
                           smlfloat& lnp,
                           smlfloat& lnp_norm,
                           const bool is_cat_post_prob_mode = false);

/// get prob of data given model
///
/// simplist call: this version creates the parameter independent data
/// for you for each call, convenient but less efficient.
///
void
get_lnprob_from_param(const subs_ml_model& mdl,
                      const site_data_fastlup& sdf,
                      smlfloat& lnp,
                      smlfloat& lnp_norm,
                      const bool is_cat_post_prob_mode = false);

#endif
