// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: prob_gtor_simple.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __PROB_GTOR_SIMPLE_H
#define __PROB_GTOR_SIMPLE_H


#include "prob_gtor.h"


/// \brief simple prob generator: 1 parameter per distro element
///
template <typename FloatType>
struct prob_gtor_simple : prob_gtor<FloatType> {

  typedef prob_gtor_simple<FloatType> self_t;
  typedef prob_gtor<FloatType> base_t;

  explicit
  prob_gtor_simple(const unsigned init_state_size = 0)
    : base_t(init_state_size,init_state_size) {}

  virtual self_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    const unsigned ps(this->param_size());
    for(unsigned i(0);i<ps;++i) this->_param[i]= this->_distro[i];
  }

  virtual
  void param_update(const bool) {
    const unsigned ps(this->param_size());
    pdistro_norm_free_param(this->_param,
                            this->_param+ps,
                            this->_is_train_param.begin());
    for(unsigned i(0);i<ps;++i) this->_distro[i] = this->_param[i];
  }
};


#endif
