// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: rate_gtor.hh 741 2007-08-14 02:39:19Z ctsa $

/// \file

#include "param_util.h"


template <typename OutputIterator>
OutputIterator
rate_gtor_base::
param_state(OutputIterator x,
            const PARAM_VIEW::index_t pv) const {
  const unsigned ps(param_size());

  if       (pv == PARAM_VIEW::ALL){
    for(unsigned i(0);i<ps;++i) { *x = _param[i]; ++x; }

  } else if(pv == PARAM_VIEW::TRAINABLE){
    for(unsigned i(0);i<ps;++i){
      if(_is_train_param[i]) {
        *x = _param[i]; ++x;
      }
    }

  } else if(pv == PARAM_VIEW::MIN_PARAM){
    const unsigned pps(pseudo_param_size());
    for(unsigned i(0);i<pps;++i) { *x = 1.; ++x; }

    for(unsigned i(0);i<ps;++i){
      if(_is_train_param[i]) {
        *x = trainable_param_transform(i);
        ++x;
      }
    }

  } else if(pv == PARAM_VIEW::INDY_MIN_PARAM){
    bool is_in_block(false);

    for(unsigned i(0);i<ps;++i){
      if(_is_free_block_start[i]) is_in_block=true;

      if(_is_train_param[i] && ! is_in_block){
        *x = trainable_param_transform(i);
        ++x;
      }

      if(_is_free_block_stop[i]) {
        is_in_block=false;

        x=dependent_to_independent_param_block(_is_train_param,
                                               _is_free_block_start,
                                               i,
                                               _param,
                                               x);
      }
    }

  } else { die("unknown param view"); }

  return x;
}




template <typename InputIterator>
InputIterator
rate_gtor_base::
set_param_state(InputIterator x,
                const PARAM_VIEW::index_t pv){
  const unsigned ps(param_size());

  if (pv == PARAM_VIEW::ALL){
    for(unsigned i(0);i<ps;++i) {
      _param[i] = *x;
      if(_param[i] < 0.) _param[i] *= -1.;
      ++x;
    }

  } else if(pv == PARAM_VIEW::TRAINABLE){
    for(unsigned i(0);i<ps;++i) {
      if(_is_train_param[i]) {
        _param[i] = *x;
        if(_param[i] < 0.) _param[i] *= -1.;
        ++x;
      }
    }

  } else if(pv == PARAM_VIEW::MIN_PARAM){
    typedef typename std::iterator_traits<InputIterator>::value_type InputT;

    const unsigned pps(pseudo_param_size());
    std::vector<InputT> p(pps);

    // 1) store pseudoparams
    for(unsigned i(0);i<pps;++i) {
      p[i] = *x;
      if(p[i] < 0.) p[i] *= -1.;
      ++x;
    }

    // 2) read regular params
    for(unsigned i(0);i<ps;++i) {
      if(_is_train_param[i]) {
        smlfloat tmp = *x;
        if(tmp < 0.) tmp *= -1.;
        trainable_param_untransform(i,tmp);
        ++x;
      }
    }

    // 3) go back to pseudoparams after regular params have been read:
    for(unsigned i(0);i<pps;++i) { set_pseudo_param(i,p[i]); }

  } else if(pv == PARAM_VIEW::INDY_MIN_PARAM){

    x=set_indy_min_param(_is_train_param,
                         _is_free_block_start,
                         _is_free_block_stop,
                         ps,
                         _param,
                         x);

    for(unsigned i(0);i<ps;++i){
      if(_is_train_param[i]) trainable_param_untransform(i,_param[i]);
    }

  } else { die("unknown param view"); }

  check_param();
  fix_input_param();
  return x;
}
