// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: sim.cc 741 2007-08-14 02:39:19Z ctsa $

/// \file

#include "rate_gtor_nuc.h"
#include "sim.h"
#include "sim_discrete.h"
#include "sim_context.h"
#include "sim_iss.h"

#include "util/general/die.h"


void
simulate_data(const sim_options& sim_opt,
              const subs_ml_model& mdl,
              site_data& sd){

  const RATE_GTOR_MODEL::index_t rgm(mdl.rate_gtor_model());
  const bool is_conditioned_site_model(RATE_GTOR_MODEL::base_overlap_size(rgm) != 0);

  const rate_gtor_nuc_base* rgn(dynamic_cast<const rate_gtor_nuc_base*>(&mdl.get_rate_gtor()));

  bool is_indy_nuc_model(true);
  if(rgn){
    for(unsigned i(0);i<mdl.get_cat_manager().get_typed_cat_size(SITE_CAT_TYPE::MUT_MODEL);++i){
      if(rgn->context_model_nuc(i) != CONTEXT_MODEL_NUC::INDY){
        is_indy_nuc_model=false;
      }
    }
  }

  if( sim_opt.method == SIM_MODEL::ISS && ! is_indy_nuc_model ){
    pass_away("iss simulation cannot be used with context-dependent mutation");
  }


  if       ( sim_opt.method == SIM_MODEL::CONTINUOUS){
    // no simpler indy mutation version exists yet:
    simulate_data_context(sim_opt,mdl,sd);


  } else if( sim_opt.method == SIM_MODEL::DISCRETE ){

    // context simulator is required for conditioned sites to properly
    // simulate the root distribution, and to properly handle partial
    // codon selection
    //
    // context simulator is also required for site model conversion
    //
    if(is_conditioned_site_model || (! is_indy_nuc_model) || sim_opt.output_site_model != rgm){
      simulate_data_context(sim_opt,mdl,sd);
    } else {
      simulate_data_discrete(sim_opt,mdl,sd);
    }

  } else if ( sim_opt.method == SIM_MODEL::ISS ){
    simulate_data_indy_site_sampler(sim_opt,mdl,sd);
  } else {
    die("unknown simulation method");
  }
}
