// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: site_data.h 741 2007-08-14 02:39:19Z ctsa $

/// \file
///
/// cat info is stored in sites
///
/// pair site info stores cat info for current and preceding site
///


#ifndef __SITE_DATA_H
#define __SITE_DATA_H

#include "count_data_code.h"
#include "name_id_lup.h"
#include "site_code.h"
#include "util/bio/bioseq_util.h"

#include <iosfwd>
#include <map>
#include <string>
#include <utility>


typedef std::pair<const count_data_code,unsigned> site_count_pair;

#ifdef NO_BOOST_POOL_ALLOCATOR
#ifdef __GNUC__
#include <ext/pool_allocator.h>
typedef __gnu_cxx::__pool_alloc<site_count_pair> site_count_alloc;
#else
typedef std::allocator<site_count_pair> site_count_alloc;
#endif
#else
#include <boost/pool/pool_alloc.hpp>
typedef boost::fast_pool_allocator<site_count_pair> site_count_alloc;
#endif

typedef std::map<count_data_code,unsigned,std::less<count_data_code>, site_count_alloc> site_count_data_type;

typedef std::map<site_code,site_count_data_type> site_count_map;


typedef unsigned char cat_type;
typedef std::pair<cat_type,cat_type> cat_pair_type;
typedef std::pair<site_code,site_code> site_pair_type;
typedef std::pair<site_pair_type,cat_pair_type> kite_pair_type;

typedef std::map<kite_pair_type,unsigned> pair_count_map;



/// \brief holds processed sequence site data
///
struct site_data {

  site_data() : sm(SITE_MODEL::NONE) {}

  unsigned n_taxa() const { return taxid.size(); }

  unsigned max_cat() const;

  // quick debugging
  void peek(std::ostream& os) const;

  void store_state(std::ostream& os,
                   const char* const cmdline = 0,
                   long int seed = -1) const;

  void load_state(std::istream& is);

  //////////////////////////////////
  std::map<std::string,unsigned> catlabel_2_id;
  std::map<unsigned,std::string> id_2_catlabel;
  name_id_lup taxid;
  site_count_map site_count;

  pair_count_map pair_count;

  SITE_MODEL::index_t sm;
};


#endif
