// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: site_data_fastlup.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __SITE_DATA_FASTLUP_H
#define __SITE_DATA_FASTLUP_H

#include "count_data_code.h"
#include "site_data_fastlup_core.h"
#include "subs_ml_types.h"

#include <vector>
#include <utility>


struct site_data_fastlup_cell : public site_data_fastlup_cell_core {

  typedef count_data_code::cat_type cat_type;

  unsigned unassigned_cat_count;
  unsigned* assigned_cat_count;
};



struct neighbor_count {

  neighbor_count(const unsigned init_id1 = 0,
                 const unsigned init_id2 = 0,
                 const unsigned init_count = 0)
    : id1(init_id1), id2(init_id2), count(init_count) {}

  bool
  operator<(const neighbor_count& right) const {
    if( id1 == right.id1 ) {
      return id2 < right.id2;
    } else {
      return id1 < right.id1;
    }
  }

  unsigned id1;
  unsigned id2;
  unsigned count;
};



/// \brief fast access site data structure -- formatted for use in the
/// lhood calculation loop
///
struct site_data_fastlup : public site_data_fastlup_core {

private:
  site_data_fastlup(const site_data_fastlup&);
  site_data_fastlup& operator=(const site_data_fastlup&);

public:

  typedef site_data_fastlup_core base_t;

  site_data_fastlup()
    : base_t(),n_cats(0),total_count(0),
      is_use_assigned_cats(false),
      is_use_unassigned_cats(false),
      data(0),assigned_cat_count_pool(0) {}

  ~site_data_fastlup() { clear(); };

  void init(unsigned init_len,
            unsigned init_n_orgs,
            unsigned init_n_cats);

  /// sort and setup for fast indexing
  void fix();

  const std::vector<bool>&
  get_cat_site_mask(const unsigned c) const {
    return cat_site_mask[c];
  }

private:
  void clear(bool is_clear_base=true);

public:
  // think about listing cat_counts in single full list, with sites,
  // so that this info can be drawn over in one pass with no zeros

  /// \todo cleanup behavior of this code for n_cat==1 case
  unsigned n_cats;

  unsigned total_count;

  bool is_use_assigned_cats;
  bool is_use_unassigned_cats;

  site_data_fastlup_cell* data;

  std::vector<neighbor_count> nb_left;
  std::vector<neighbor_count> nb_right;

  std::vector<std::vector<bool> > cat_site_mask;

private:
  unsigned* assigned_cat_count_pool;
};


#endif
