// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: subs_ml_minfunc.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __SUBS_ML_MINFUNC_H
#define __SUBS_ML_MINFUNC_H


#include "estep_dat_t.h"
#include "lhood_model.h"
#include "lhood_model_em.h"
#include "site_data_fastlup.h"
#include "subs_ml_model.h"
#include "util/math/minfunc_interface.h"


extern unsigned min_call_count;
extern unsigned dmin_call_count;


struct subs_ml_minfunc_base : public minfunc_gradient_interface<smlfloat> {

  subs_ml_minfunc_base(subs_ml_model& init_mdl,
                       PARAM_VIEW::index_t init_pv)
    : mdl(init_mdl), pv(init_pv) {}

  virtual
  unsigned dim() const { return mdl.param_size(pv); }

  void min_p_to_param(const smlfloat* p,
                      const bool is_secondary_update=true) {
    min_p_to_param_x(p,mdl,is_secondary_update);
  }

  void min_p_to_param_x(const smlfloat* p,
                        subs_ml_model& mdl_copy,
                        const bool is_secondary_update=true) const {
    mdl_copy.set_param_state(p,pv,is_secondary_update);
  }

  void param_to_min_p(smlfloat* p) const {
    mdl.param_state(p,pv);
  }

  virtual void get_lnprob(const subs_ml_model& mdl_copy,
                          smlfloat& lnp,
                          smlfloat& lnp_norm) const = 0;

  virtual bool is_inner_step() const = 0;


  virtual
  smlfloat dval(const smlfloat* p,smlfloat* df);

  virtual
  smlfloat val(const smlfloat* p);

  virtual
  bool is_val_computable(const smlfloat* p){
    min_p_to_param(p,false);
    return mdl.param_penalty() <= 0. && (!mdl.get_time_gtor().is_zero_time());
  }

private:
  smlfloat get_deriv_estimate(const smlfloat* p,
                              const smlfloat lnp1_norm,
                              const unsigned param_no,
                              const bool is_plus = true) const;
protected:
  subs_ml_model& mdl;
private:
  PARAM_VIEW::index_t pv;
};




struct subs_ml_minfunc : public subs_ml_minfunc_base {

  subs_ml_minfunc(const site_data_fastlup& init_sdf,
                  subs_ml_model& init_mdl,
                  const PARAM_VIEW::index_t init_pv)
    : subs_ml_minfunc_base(init_mdl,init_pv), sdf(init_sdf),
      is_ppit(init_mdl.opt().is_ppit), nld(0) {
    if(! is_ppit) nld = new nonparam_lnprob_data(init_mdl,init_sdf);
  }

  ~subs_ml_minfunc() { if(nld) delete nld; }

  virtual
  void get_lnprob(const subs_ml_model& mdl_copy,
                  smlfloat& lnp,
                  smlfloat& lnp_norm) const {
    if(is_ppit){
      get_lnprob_from_param(mdl_copy,sdf,lnp,lnp_norm);
    } else {
      get_lnprob_from_param_prep(mdl_copy,sdf,*nld,lnp,lnp_norm);
    }
  }

  virtual
  bool is_inner_step() const { return false; }

private:
  const site_data_fastlup& sdf;
  const bool is_ppit;
  nonparam_lnprob_data* nld;
};



struct subs_ml_minfunc_em : public subs_ml_minfunc_base {

  subs_ml_minfunc_em(const estep_dat_t& init_edat,
                     subs_ml_model& init_mdl,
                     const PARAM_VIEW::index_t init_pv)
    : subs_ml_minfunc_base(init_mdl,init_pv), edat(init_edat) {}

  virtual
  void get_lnprob(const subs_ml_model& mdl_copy,
                  smlfloat& lnp,
                  smlfloat& lnp_norm) const {
    get_lnprob_from_param(mdl_copy,edat,lnp,lnp_norm);
  }

  virtual
  bool is_inner_step() const { return true; }

private:
  const estep_dat_t& edat;
};



#endif
