// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: io_util.cc 741 2007-08-14 02:39:19Z ctsa $

/// \file

#include "log.h"

#include <cstdlib>

#include <iostream>

void
check_nonempty_istream(std::istream& is,
                       const char* stream_label){

  // rudimentary stream check
  if( ! is ) {
    log_os << "ERROR:: invalid input stream: " << stream_label << "\n";
    exit(EXIT_FAILURE);
  }
  is.get();
  if( is.eof() ){
    log_os << "ERROR:: input stream is empty: " << stream_label << "\n";
    exit(EXIT_FAILURE);
  }
  is.unget();
}


// clear out ws, stop after newline
void
clear_whitespace_line(std::istream& is){
  while(is){
    const int c(is.get());
    if(c==' ' || c== '\t') continue;

    if(c!='\n') is.unget();
    break;
  }
}

// clear to newline
void
clear_line(std::istream& is){
  while(is){
    const int c(is.get());

    if(c=='\n') { break;}
  }
}
